package com.hungraim.ltc.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.SysConfig.ChCfgSrvOrganType;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndSrvOrganLevel;
import com.hungraim.ltc.pojo.entity.disable.ChServiceType;
import com.hungraim.ltc.pojo.entity.disable.ChServiceTypeDetailed;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrganApply;
import com.hungraim.ltc.pojo.entity.riskLevel.ChSrvorganRiskLevel;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganProject;
import com.hungraim.ltc.pojo.vo.disable.ChServiceTypeVo;
import com.hungraim.ltc.pojo.vo.disable.SrvModeVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganListVo;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.pojo.vo.system.SrvOrganVo;
import com.hungraim.ltc.service.ChSrvOrganProjectService;
import com.hungraim.ltc.service.OrganService;
import com.hungraim.ltc.service.SrvOrganService;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@AllArgsConstructor
@Slf4j
public class SrvOrganServiceImpl implements SrvOrganService {

    private final SrvOrganMapper srvOrganMapper;

    private final SrvOrganApplyMapper srvOrganApplyMapper;


    private final OrganService organService;

    private final SrvModeMapper srvModeMapper;

    private final ChFndSrvOrganLevelMapper chFndSrvOrganLevelMapper;

    private final ChCfgSrvOrganTypeMapper chCfgSrvOrganTypeMapper;

    private final AttachFeignService attachFeignService;

    private final ChSrvorganRiskLevelMapper chSrvorganRiskLevelMapper;

    private final ServiceTypeMapper serviceTypeMapper;
    private final ServiceTypeDetailedMapper  serviceTypeDetailedMapper;
    private final ChSrvOrganProjectMapper  chSrvOrganProjectMapper;
    private final ChSrvOrganProjectService chSrvOrganProjectService;



    @Override
    public Page<SrvOrganListVo> getSrvOrganList(Page<SrvOrganListVo> page, Long organId, Boolean isInvolveChild, String creditCode, String srvOrganName,Long srvOrganId) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        Page<SrvOrganListVo> srvOrganList = srvOrganMapper.getSrvOrganList(page, organIds, creditCode, srvOrganName,srvOrganId);
        List<SrvOrganListVo> records = srvOrganList.getRecords();
        if (records != null && records.size() > 0) {
            for (SrvOrganListVo srvOrganListVo : records) {
                String srvType = srvOrganListVo.getSrvType();
                if (StrUtil.isNotBlank(srvType)) {
                    String[] split = srvType.split(",");
                    List<Integer> modeIds = new ArrayList<>();
                    for (String s : split) {
                        modeIds.add(Integer.parseInt(s));
                    }
                    // 去查询对应的服务方式
                    List<SrvModeVo> srvModeVos = srvModeMapper.selectByModeIds(modeIds);
                    srvOrganListVo.setSrvModeVos(srvModeVos);
                }
            }
        }
        return srvOrganList;
    }


    @Override
    public Page<SrvOrganListVo> getSrvOrganApplyList(Page<SrvOrganListVo> page, Long organId, Boolean isInvolveChild, String creditCode, String srvOrganName, Long srvOrganId) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        Page<SrvOrganListVo> srvOrganApplyList = srvOrganApplyMapper.getSrvOrganApplyList(page, organIds, creditCode, srvOrganName,srvOrganId);
        List<SrvOrganListVo> records = srvOrganApplyList.getRecords();
        if (records != null && records.size() > 0) {
            for (SrvOrganListVo srvOrganListVo : records) {
                String srvType = srvOrganListVo.getSrvType();
                if (StrUtil.isNotBlank(srvType)) {
                    String[] split = srvType.split(",");
                    List<Integer> modeIds = new ArrayList<>();
                    for (String s : split) {
                        modeIds.add(Integer.parseInt(s));
                    }
                    // 去查询对应的服务方式
                    List<SrvModeVo> srvModeVos = srvModeMapper.selectByModeIds(modeIds);
                    srvOrganListVo.setSrvModeVos(srvModeVos);
                }
            }
        }
        return srvOrganApplyList;
    }

    @Override
    public List<ChFndSrvOrganLevel> getSrvOrganLevels() {
        return chFndSrvOrganLevelMapper.selectList(null);
    }

    @Override
    public List<ChCfgSrvOrganType> getSrvOrganTypes() {
        return chCfgSrvOrganTypeMapper.selectList(null);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateSrvOrgan(SrvOrganVo srvOrganVo) {
        Integer srvOrganId = srvOrganVo.getSrvOrganId();
        List<ChFndAttach> fileDataList = srvOrganVo.getFileDataList();
        ChSrvOrganApply chSrvOrganApply = new ChSrvOrganApply();
        BeanUtils.copyProperties(srvOrganVo, chSrvOrganApply);
        if (srvOrganId != null) {
            // 修改操作
            srvOrganApplyMapper.updateById(chSrvOrganApply);
            AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
            attachUpdateVo.setCtrlId(Long.valueOf(srvOrganId));
            attachUpdateVo.setAttachType(AttachType.SRV_ORGAN.getUploadType());
            attachUpdateVo.setChFndAttachs(fileDataList);
            Result attachs = attachFeignService.updateAttachs(attachUpdateVo);
            if (!StrUtil.equals(attachs.getCode(), Result.success().getCode())) {
                log.error("同步修改文件数据失败，失败原因:{}", attachs.getMsg());
            }
        } else {
            // 添加操作
            //校验服务机构是否已存在
            ChSrvOrganApply selectOrgan = srvOrganApplyMapper.selectOne(new LambdaQueryWrapper<ChSrvOrganApply>()
                    .eq(ChSrvOrganApply::getUnifiedSocialCreditCode, srvOrganVo.getUnifiedSocialCreditCode())
                    .eq(ChSrvOrganApply::getStatus, 0));
            ChSrvOrganApply selectOrgan1 = srvOrganApplyMapper.selectOne(new LambdaQueryWrapper<ChSrvOrganApply>()
                    .eq(ChSrvOrganApply::getPermitId, srvOrganVo.getPermitId())
                    .eq(ChSrvOrganApply::getStatus, 0));
            if (selectOrgan != null || selectOrgan1 != null) {
                throw new RuntimeException("服务机构已存在,请不要重复添加");
            }
            chSrvOrganApply.setStatus(0);
            chSrvOrganApply.setEffTime(new Date());
            srvOrganApplyMapper.insert(chSrvOrganApply);
            AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
            attachUpdateVo.setCtrlId(Long.valueOf(chSrvOrganApply.getSrvOrganId()));
            attachUpdateVo.setAttachType(AttachType.SRV_ORGAN.getUploadType());
            attachUpdateVo.setChFndAttachs(fileDataList);
            Result attachs = attachFeignService.updateAttachs(attachUpdateVo);
            if (!StrUtil.equals(attachs.getCode(), ResultCode.SUCCESS.getCode())) {
                log.error("同步修改文件数据失败，失败原因:{}", attachs.getMsg());
            }
            ChSrvorganRiskLevel chSrvorganRiskLevel = new ChSrvorganRiskLevel();
            chSrvorganRiskLevel.setRiskLevel(0L);
            chSrvorganRiskLevel.setSrvOrganId(Long.valueOf(chSrvOrganApply.getSrvOrganId()));
            chSrvorganRiskLevel.setStatus(0);
            chSrvorganRiskLevel.setFcd(new Date());
            chSrvorganRiskLevelMapper.insert(chSrvorganRiskLevel);

            //添加48项服务配置
            //1、查询所有的服务配置
            List<ChServiceType> list = serviceTypeMapper.selectList(new QueryWrapper<>());
            List<ChServiceTypeVo> chServiceTypeVoList = new ArrayList<ChServiceTypeVo>();
            for(ChServiceType chServiceType : list){
                if(chServiceType.getParaId() == 0L){
                    ChServiceTypeVo chServiceTypeVo = new ChServiceTypeVo();
                    chServiceTypeVo.setServiceTypeId(chServiceType.getServiceTypeId());
                    chServiceTypeVo.setServiceTypeName(chServiceType.getServiceTypeName());
                    QueryWrapper<ChServiceTypeDetailed> wrapper = new QueryWrapper<ChServiceTypeDetailed>();
                    wrapper.eq("para_id", chServiceType.getServiceTypeId());
                    List<ChServiceTypeDetailed> detailedList = serviceTypeDetailedMapper.selectList(wrapper);
                    chServiceTypeVo.setList(detailedList);
                    chServiceTypeVoList.add(chServiceTypeVo);
                }
            }
            Long srvOrganIdL = Long.valueOf(chSrvOrganApply.getSrvOrganId());
            List<ChSrvOrganProject> chSrvOrganProjectList = new ArrayList<>();
            //进行服务处理
            chServiceTypeVoList.stream().forEach(chServiceTypeVo->{
                chServiceTypeVo.getList().stream().forEach(e->{
                    ChSrvOrganProject chSrvOrganProject = new ChSrvOrganProject();
                    chSrvOrganProject.setSrvOrganId(srvOrganIdL);
                    chSrvOrganProject.setItemName(e.getServiceTypeDetailedId()+"");
                    chSrvOrganProject.setCategoryType(chServiceTypeVo.getServiceTypeId().shortValue());
                    chSrvOrganProject.setIsConf(new Short("0"));
                    chSrvOrganProject.setStatus(new Short("0"));
                    chSrvOrganProjectList.add(chSrvOrganProject);
                });
            });
            //批量添加内容
            chSrvOrganProjectService.saveBatch(chSrvOrganProjectList);
        }
    }

    @Override
    public SrvOrganVo getSrvOrganDetail(Long srvOrganId) {
        ChSrvOrgan chSrvOrgan = srvOrganMapper.selectById(srvOrganId);
        SrvOrganVo srvOrganVo = new SrvOrganVo();
        BeanUtils.copyProperties(chSrvOrgan, srvOrganVo);
        try {
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(srvOrganId, AttachType.SRV_ORGAN.getUploadType());
            if (StrUtil.equals(listResult.getCode(), ResultCode.SUCCESS.getCode())) {
                List<ChFndAttach> data = listResult.getData();
                if (data != null) {
                    srvOrganVo.setFileDataList(data);
                }
            } else {
                log.error("查询id为{}的护理机构文件列表失败，失败原因：{}", srvOrganId, listResult.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询id为{}的护理机构文件列表失败，失败原因：{}", srvOrganId, e.getMessage());
        }
        return srvOrganVo;
    }


    @Override
    public SrvOrganVo getSrvOrganApplyDetail(Long srvOrganId) {
        ChSrvOrganApply chSrvOrganApply = srvOrganApplyMapper.selectById(srvOrganId);
        SrvOrganVo srvOrganVo = new SrvOrganVo();
        BeanUtils.copyProperties(chSrvOrganApply, srvOrganVo);
        try {
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(srvOrganId, AttachType.SRV_ORGAN.getUploadType());
            if (StrUtil.equals(listResult.getCode(), ResultCode.SUCCESS.getCode())) {
                List<ChFndAttach> data = listResult.getData();
                if (data != null) {
                    srvOrganVo.setFileDataList(data);
                }
            } else {
                log.error("查询id为{}的护理机构文件列表失败，失败原因：{}", srvOrganId, listResult.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询id为{}的护理机构文件列表失败，失败原因：{}", srvOrganId, e.getMessage());
        }
        return srvOrganVo;
    }

    @Override
    public void deleteSrvOrgan(Long srvOrganId) {
        ChSrvOrgan chSrvOrgan = new ChSrvOrgan();
        chSrvOrgan.setSrvOrganId(srvOrganId.intValue());
        chSrvOrgan.setStatus(1);
        ChSrvOrganApply chSrvOrganApply = new ChSrvOrganApply();
        chSrvOrganApply.setSrvOrganId(srvOrganId.intValue());
        chSrvOrganApply.setStatus(1);
        srvOrganMapper.updateById(chSrvOrgan);
        srvOrganApplyMapper.updateById(chSrvOrganApply);
    }

    @Override
    public void auditSrvOrgan(Integer userId,Long srvOrganId, Integer auditStatus) {
        ChSrvOrganApply chSrvOrganApply = new ChSrvOrganApply();
        chSrvOrganApply.setSrvOrganId(srvOrganId.intValue());
        chSrvOrganApply.setAuditStatus(auditStatus.shortValue());
        chSrvOrganApply.setAuditDate(new Date());
        chSrvOrganApply.setAuditUserId(userId);
        int result = srvOrganApplyMapper.updateById(chSrvOrganApply);
        //审核通过 同步到主表
        if (result>0) {
            //查出申请表的信息同步到主表
            chSrvOrganApply = srvOrganApplyMapper.selectOne(new LambdaQueryWrapper<ChSrvOrganApply>()
                    .eq(ChSrvOrganApply::getSrvOrganId,srvOrganId));
            //主表
            ChSrvOrgan querySrvOrgan = srvOrganMapper.selectOne(new LambdaQueryWrapper<ChSrvOrgan>()
                    .eq(ChSrvOrgan::getSrvOrganId,srvOrganId));
            ChSrvOrgan chSrvOrgan = new ChSrvOrgan();
            BeanUtils.copyProperties(chSrvOrganApply,chSrvOrgan);
            if(querySrvOrgan==null){
                srvOrganMapper.insert(chSrvOrgan);
            }else{
                srvOrganMapper.updateById(chSrvOrgan);
            }
        }
    }
}
